import * as nps from 'path';
import { BaseJavaCstVisitorWithDefaults, parse } from 'java-parser';
import { sep, dirname } from 'path';
class ParseFileMeta extends BaseJavaCstVisitorWithDefaults {
    constructor(options = {}) {
        super();
        this.options = options;
        this.data = {
            type: null,
            access: 'default',
            package: null,
            name: ''
        };
        this.state = {
            inClassDeclaration: false,
            done: false
        };
        const { classPath, filename } = this.options;
        if (filename) {
            const relativeFilename = classPath ? nps.relative(classPath, filename) : filename;
            this.data.package = dirname(relativeFilename).split(sep);
            this.data.name = nps.basename(relativeFilename, nps.extname(relativeFilename));
        }
    }
    packageDeclaration(ctx) {
        var _a;
        if ((_a = ctx.Identifier) === null || _a === void 0 ? void 0 : _a.length) {
            this.data.package = ctx.Identifier.map((x) => x.image);
        }
    }
    classDeclaration(ctx, param) {
        var _a, _b;
        if (this.state.done || this.state.inClassDeclaration) {
            return;
        }
        if (ctx.enumDeclaration) {
            this.data.type = 'enum';
        }
        else if (ctx.recordDeclaration) {
            this.data.type = 'record';
        }
        else if (ctx.normalClassDeclaration) {
            this.data.type = 'class';
        }
        const classModifierCtx = (_b = (_a = ctx.classModifier) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.children;
        if (classModifierCtx) {
            if (classModifierCtx.Public) {
                this.data.access = 'public';
            }
            else if (classModifierCtx.Protected) {
                this.data.access = 'protected';
            }
            else if (classModifierCtx.Private) {
                this.data.access = 'private';
            }
            // classModifierCtx.Abstract
            // classModifierCtx.Final
        }
        this.state.inClassDeclaration = true;
        return super.classDeclaration(ctx, param);
    }
    typeIdentifier(ctx, param) {
        var _a;
        if (!this.state.done && this.state.inClassDeclaration) {
            this.state.done = true;
            this.data.name = ((_a = ctx.Identifier[0]) === null || _a === void 0 ? void 0 : _a.image) || this.data.name;
        }
    }
}
export function parseFileMeta(contentOrAst, opts) {
    const v = new ParseFileMeta(opts);
    v.visit(typeof contentOrAst === 'string' ? parse(contentOrAst) : contentOrAst);
    return v.data;
}
