var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { BaseJavaCstVisitorWithDefaults } from 'java-parser';
import { parseImport } from './parse-imports';
import { getExpressionValue } from './parse-enum-code';
/**
 * 解析
 * class class X {
 *   public static int a = 123;
 *   private boolean b = true;
 * }
 * // => { a: { type: 'primitive', value: 123, raw: '123' }, b: { type: 'primitive', value: true, raw: 'true' } }
 */
class ParseBodyMember extends BaseJavaCstVisitorWithDefaults {
    constructor() {
        super(...arguments);
        this.data = new Map();
        this.state = {
            inClassMember: false
        };
    }
    fieldDeclaration(ctx) {
        this.state.inClassMember = true;
        super.classMemberDeclaration(ctx);
        this.state.inClassMember = false;
        const isStatic = !!ctx.fieldModifier.find((x) => x.children.Static);
        const tmpCtx = ctx.variableDeclaratorList[0].children.variableDeclarator[0].children;
        if (tmpCtx.variableInitializer && tmpCtx.variableDeclaratorId && tmpCtx.Equals) {
            const value = getExpressionValue(tmpCtx.variableInitializer[0].children.expression[0]);
            if (value) {
                this.data.set(tmpCtx.variableDeclaratorId[0].children.Identifier[0].image, {
                    value,
                    static: isStatic
                });
            }
        }
    }
}
export function parseBodyMember(ast) {
    const v = new ParseBodyMember();
    v.visit(ast);
    return v.data;
}
/**
 * 解决嵌套，动态计算的值
 * @param ast
 * @param getAstMapByPackage
 * @param getAstById
 * @param resolveExpressionValue
 * @param map
 */
export const resolveImportsValue = (ast, { getAstMapByPackage, getAstById, map = new Map(), resolveExpressionValue }) => {
    if (!ast) {
        return null;
    }
    const imports = parseImport(ast);
    const scopedMap = new Map();
    const getMemberMap = (id) => {
        const newAst = getAstById(id);
        if (!newAst) {
            return;
        }
        if (map.get(id)) {
            return map.get(id);
        }
        const memberMap = parseBodyMember(newAst);
        if (resolveExpressionValue) {
            memberMap.forEach((item, key) => {
                item.value = resolveExpressionValue(item.value, id, key);
            });
        }
        map.set(id, memberMap);
        return memberMap;
    };
    const getMemberMapByPackage = (id) => {
        const result = new Map();
        const newAsts = getAstMapByPackage(id);
        if (!newAsts) {
            return result;
        }
        Object.keys(newAsts).forEach((name) => {
            const classid = [id].concat(name).join('.');
            result.set(name, getMemberMap(classid));
        });
        return result;
    };
    imports.forEach((x) => {
        var _a;
        const id = x.package ? x.package.concat(x.name).filter(Boolean).join('.') : x.name;
        if (!id) {
            return;
        }
        if (x.wildcard) {
            if (x.name) {
                // import java.io.Util.*;
                const memberMap = getMemberMap(id);
                if (memberMap) {
                    for (const [name, item] of memberMap.entries()) {
                        if (item.static) {
                            scopedMap.set(name, item);
                        }
                    }
                }
            }
            else {
                // import java.io.*;
                const packageMap = getMemberMapByPackage(id);
                if (packageMap) {
                    for (const [name, item] of packageMap.entries()) {
                        scopedMap.set(name, item);
                    }
                }
            }
            return;
        }
        if (x.name) {
            // import java.io.Util.a;
            if ((_a = x.member) === null || _a === void 0 ? void 0 : _a.length) {
                if (x.member.length === 1) {
                    const memberMap = getMemberMap(id);
                    if (memberMap && memberMap.get(x.member[0])) {
                        scopedMap.set(x.member[0], memberMap.get(x.member[0]));
                    }
                }
            }
            else {
                // import java.io.Util;
                const memberMap = getMemberMap(id);
                if (memberMap) {
                    scopedMap.set(x.name, memberMap);
                }
            }
        }
    });
    return scopedMap;
};
export function resolveExpressionValue(item, _a) {
    var _b, _c, _d;
    var { _visitMap = new Map(), id, fieldName } = _a, opts = __rest(_a, ["_visitMap", "id", "fieldName"]);
    const options = Object.assign({ _visitMap,
        id,
        fieldName }, opts);
    if (item.type === 'fqnOrRefType') {
        const fieldKey = `${id}#${fieldName}`;
        const prevData = _visitMap.get(fieldKey);
        if ((prevData === null || prevData === void 0 ? void 0 : prevData.status) === 'post') {
            return prevData.data;
        }
        if ((prevData === null || prevData === void 0 ? void 0 : prevData.status) === 'pre') {
            throw new Error(`存在环形依赖解析 ${fieldKey}`);
        }
        _visitMap.set(fieldKey, { status: 'pre' });
        const importsValue = resolveImportsValue(opts.getAstById(id), Object.assign(Object.assign({}, opts), { resolveExpressionValue: (item, id, key) => {
                return resolveExpressionValue(item, Object.assign(Object.assign({}, options), { fieldName: key, id }));
            } }));
        if (!importsValue) {
            _visitMap.set(fieldKey, { status: 'post', data: item });
            return item;
        }
        if ((_b = item.value) === null || _b === void 0 ? void 0 : _b.length) {
            let map = importsValue;
            for (let i = 0; i < item.value.length; i++) {
                map = map.get(item.value[i]);
                if (!(map instanceof Map)) {
                    if (i === item.value.length - 1) {
                        _visitMap.set(fieldKey, { status: 'post', data: (_c = map === null || map === void 0 ? void 0 : map.value) !== null && _c !== void 0 ? _c : item });
                        return (_d = map === null || map === void 0 ? void 0 : map.value) !== null && _d !== void 0 ? _d : item;
                    }
                    break;
                }
            }
        }
    }
    return item;
}
