"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.resultItemToCode = exports.resultToCode = exports.javaEnumTransformByDir = void 0;
/**
 * Java enum transform
 * @author 余聪
 */
const globby = require("globby");
const fsExtra = require("fs-extra");
const change_case_1 = require("change-case");
const java_parser_1 = require("java-parser");
const parse_enum_code_1 = require("./parse-enum-code");
const parse_file_1 = require("./parse-file");
const resolve_values_1 = require("./resolve-values");
function javaEnumTransformByDir(dir, { ignore = [], pattern = '**/*.java', deRef = true } = {}) {
    return __awaiter(this, void 0, void 0, function* () {
        const filenames = yield globby(pattern, {
            absolute: true,
            onlyFiles: true,
            ignore,
            cwd: dir
        });
        const tmpFileMap = new Map();
        const fileMap = new Map();
        yield Promise.all(filenames.map((filename) => __awaiter(this, void 0, void 0, function* () {
            const code = yield fsExtra.readFile(filename, 'utf-8');
            const ast = (0, java_parser_1.parse)(code);
            const meta = (0, parse_file_1.parseFileMeta)(ast, { filename, classPath: dir });
            const id = meta.package ? meta.package.concat(meta.name).join('.') : null;
            if (id) {
                tmpFileMap.set(id, {
                    id,
                    meta,
                    ast
                });
            }
        })));
        const sortedKeys = Array.from(tmpFileMap.keys()).sort();
        sortedKeys.forEach((name) => {
            fileMap.set(name, tmpFileMap.get(name));
            tmpFileMap.delete(name);
        });
        const deRefOptions = {
            _visitMap: new Map(),
            getAstMapByPackage: (prefix) => {
                const map = {};
                const ids = [...fileMap.keys()];
                ids.forEach((id) => {
                    const namespace = prefix + '.';
                    if (id.startsWith(namespace)) {
                        const name = id.slice(namespace.length);
                        if (!name.includes('.') && fileMap.get(id)) {
                            map[name] = fileMap.get(id).ast;
                        }
                    }
                });
                return map;
            },
            getAstById: (id) => {
                var _a;
                return (_a = fileMap.get(id)) === null || _a === void 0 ? void 0 : _a.ast;
            },
            map: new Map()
        };
        function deRefFn(list, file) {
            const resolveExpressionValueInner = (item, fieldName) => {
                if (!item) {
                    return;
                }
                try {
                    return (0, resolve_values_1.resolveExpressionValue)(item, Object.assign(Object.assign({}, deRefOptions), { id: file.id, fieldName }));
                }
                catch (e) {
                    e.message = `解析 Enum 数据出错, id: ${file.id}, enum: ${fieldName}
  ${e.message}`;
                    throw e;
                }
            };
            list.forEach((item) => {
                item.enums.forEach((enumItem) => {
                    if (enumItem.label) {
                        enumItem.label = resolveExpressionValueInner(enumItem.label, enumItem.name);
                    }
                    if (enumItem.value) {
                        enumItem.value = resolveExpressionValueInner(enumItem.value, enumItem.name);
                    }
                });
            });
        }
        const result = new Map();
        yield Promise.all(Array.from(fileMap.entries())
            .filter(([, file]) => file.meta.type === 'enum')
            .map(([pkgName, file]) => __awaiter(this, void 0, void 0, function* () {
            const list = (0, parse_enum_code_1.parseEnumCode)(file.ast);
            if (deRef) {
                deRefFn(list, file);
            }
            result.set(pkgName, list);
        })));
        return result;
    });
}
exports.javaEnumTransformByDir = javaEnumTransformByDir;
function resultToCode(map) {
    let mergedList = [];
    for (const [, list] of map.entries()) {
        mergedList = mergedList.concat(list);
    }
    const cache = new Map();
    return (0, parse_enum_code_1.formatToTs)(mergedList, {
        transformEnumName: (name) => {
            if (!/enum$/i.test(name)) {
                name = (0, change_case_1.pascalCase)(name + ' enum');
            }
            if (!cache.has(name)) {
                cache.set(name, 0);
                return name;
            }
            cache.set(name, cache.get(name) + 1);
            return `${name}${cache.get(name)}`;
        }
    });
}
exports.resultToCode = resultToCode;
/**
 * 单个文件独立渲染
 * @param item
 * @returns
 */
function resultItemToCode(item) {
    return (0, parse_enum_code_1.formatToTs)(item, {
        transformEnumName: (name) => {
            if (!/enum$/i.test(name)) {
                name = (0, change_case_1.pascalCase)(name + ' enum');
            }
            return name;
        }
    });
}
exports.resultItemToCode = resultItemToCode;
